#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#define F_CPU 4000000UL

#include <avr/delay.h>

#include <stdio.h>

#define SetBit(port,bit) port |=  _BV(bit)
#define ClrBit(port,bit) port &= ~_BV(bit)


#define Port4Out(port,bit) port |=  _BV(bit)
#define Port4Inp(port,bit) port &= ~_BV(bit)

#define HiBYTE(w) (uint8_t)((w >> 8) & 0x00ff)
#define LoBYTE(w) (uint8_t)((w >> 0) & 0x00ff)

#define Nop() __asm__ __volatile__ ("nop" ::)

#define CONST__GAINoff 0B10001000
#define CONST__GAINon  0B10001001 // Weak Signal Booster ON
#define CONST__LoBAND  0B00000001
#define CONST__MiBAND  0B00000010
#define CONST__HiBAND  0B00001100

#define RCVPCH1 37300
#define RCVPCH2 10700

#define ADR_EE_RC5    0x01A0  // 18* 2 Bytes
#define ADR_EE_vMax   0x01F0  // 1 * 1 Bytes
#define ADR_EE_vMin   0x01F1  // 1 * 1 Bytes
#define ADR_EE_vWrk   0x01F2  // 1 * 1 Bytes
#define ADR_EE_Lo     0x01F8  // 1 * 4 Bytes
#define ADR_EE_Hi     0x01FC  // 1 * 4 Bytes

#define VolumeMAX   110
#define VolumeMIN    50

////////////////////////////
// Port Extention 74HC595 //

#define EP0 0
#define EP1 1
#define EP2 2
#define EP3 3
#define EP4 4
#define EP5 5
#define EP6 6
#define EP7 7

#define LM7001_BitClc       EP5
#define LM7001_BitDat       EP6
#define LM7001_BitCEN       EP7


#define I2C_Port_ini   DDRC
#define I2C_Port      PORTC
#define I2C_BitClc      PD5
#define I2C_BitDat      PD4


#define BUS_Port_ini   DDRD
#define BUS_Port      PORTD
#define BUS_BitClc      PD5
#define BUS_BitDat      PD6

#define TIC_Port_ini   DDRD
#define TIC_Port      PORTD
#define TIC_Load        PD7

#define Ext_Port_ini   DDRB
#define Ext_Port      PORTB
#define Ext_Load        PB0

#define KBD_Port_ini   DDRB
#define KBD_Port_Inp   PINB
#define KBD_Port      PORTB
#define KBD_BitInp1     PB3
#define KBD_BitInp2     PB4

#define KBD_0    0
#define KBD_1    1
#define KBD_2    2
#define KBD_3    3
#define KBD_4    4
#define KBD_5    5
#define KBD_6    6
#define KBD_7    7
#define KBD_8    8
#define KBD_9    9
#define KBD_F    10
#define KBD_Clr  11
#define KBD_Pls  12
#define KBD_Mns  13
#define KBD_Sql  14
#define KBD_Vol  15

#define IND_0     4
#define IND_1     0
#define IND_2     3
#define IND_3     5
#define IND_4     8
#define IND_5    11
#define IND_6    13
#define IND_7     9
#define IND_8    12
#define IND_9    14
#define IND_F     1
#define IND_Clr   6
#define IND_Pls  10
#define IND_Mns  15
#define IND_Sql   2
#define IND_Vol   7

uint8_t Ind2KBD[16]={KBD_1,KBD_F,KBD_Sql,KBD_2,
		     KBD_0,KBD_3,KBD_Clr,KBD_Vol,
		     KBD_4,KBD_7,KBD_Pls,KBD_5,
		     KBD_8,KBD_6,KBD_9,  KBD_Mns};

#define WFM  0
#define NFM  1
#define  AM  2

enum { UP, DOWN };

enum { RADIO,
       STEP,
       GAIN,
       UAPY,
       FINPUT,
       WR_H_L,
       WR_MEM,
       RC_MEM,
       SCAN_F,
       SCAN_M,
       SET_SQL,
       SET_VOL
     };

#define  NUM_MEM 100

#define  LcdBufLEN 9
#define  InpBufLEN 7

#define  RxInpBufLEN 16

#define  PROMPT "\n>"
#define  ERROR  "\nErr\n>"

volatile uint8_t RcInpBuf[RxInpBufLEN];
volatile uint8_t RxInpBuf[RxInpBufLEN];
volatile uint8_t RxInd = 0;
volatile uint8_t TxInd = 0;
volatile uint8_t RcInd = 0;

volatile uint8_t EchoYes = 1;

volatile uint8_t InpBuf[InpBufLEN];
	 uint8_t cccBuf[8];

volatile uint8_t InpInd;
volatile int8_t  IndMEM = 0;

	 uint8_t Step[8]={5,10,15,20,25,30,50,100};

volatile uint32_t FrcvWork;
volatile uint8_t  FrcvStep = 10;
volatile uint8_t  FindStep =  1;

volatile uint8_t  direction = UP;
volatile uint8_t  Regim;

volatile uint8_t  Modulation;

volatile uint8_t  FlagMute;
volatile uint8_t  FlagSQL = 1;
volatile uint16_t MuteCnt = 0;

volatile uint8_t Pwm1;
volatile uint8_t Pwm2;

volatile uint8_t Vol_Max;
volatile uint8_t Vol_Min;


#define  AUTO_CLR     7  // sec

volatile uint8_t AutoClr;
volatile uint8_t TimeClr;
volatile uint8_t TimeTmr;


#define  MUTE     50

#define  RCV_MAX  855000
#define  RCV_MIN   45000


volatile uint32_t FrcvMax;
volatile uint32_t FrcvMin;

	 uint8_t  FlagScanF = 0;
	 uint8_t  FlagScanM = 0;
	 uint8_t  FlagBegin = 0;

	  int8_t  ScanMBank = 0;
	 uint8_t  FlagBank;


volatile uint16_t FrcvPCH2;
volatile uint8_t  flagBo;

volatile uint8_t  Smeter;

volatile uint8_t  TunerGain  = 0;


static  void KeyNop  (void);
	void KeyMOD  (void);
	void KeyPlus (void);
	void KeyMinus(void);
	void KeyRADIO(void);
	void KeySTEP (void);
	void KeyGAIN (void);
	void KeyUapy (void);
	void KeyFinp (void);
	void KeySetHi(void);
	void KeySetLo(void);
	void KeyWrMem(void);
	void KeyRcMem(void);
	void KeyScanF(void);
	void KeyScanM(void);

	void KeySQL(void);
	void KeyVOL(void);

	void Key_0   (void);
	void Key_1   (void);
	void Key_2   (void);
	void Key_3   (void);
	void Key_4   (void);
	void Key_5   (void);
	void Key_6   (void);
	void Key_7   (void);
	void Key_8   (void);
	void Key_9   (void);

void (* SwKey[32])(void) =
     {
	KeyMOD,
	KeySTEP,
	KeyGAIN,
	KeyUapy,
	KeySetHi,
	KeyWrMem,
	KeySetLo,
	KeyScanF,
	KeyRcMem,
	KeyScanM,
	KeyFinp,
	KeyRADIO,
	KeyPlus,
	KeyMinus,
	KeySQL,
	KeyVOL,

	Key_0,
	Key_1,
	Key_2,
	Key_3,
	Key_4,
	Key_5,
	Key_6,
	Key_7,
	Key_8,
	Key_9,
	KeyFinp,
	KeyRADIO,
	KeyNop,
	KeyNop,
	KeyNop,
	KeyNop
     };



#define Tic_Comma 0x80
#define Tic_P     0b00011111
#define Tic_S     0b01101110
#define Tic_T     0b01000111
#define Tic_E     0b01001111
#define Tic_N     0b00100011
#define Tic_G     0b01101101
#define Tic_A     0b00111111
#define Tic_c     0b01000011
#define Tic_q     0b00111110
#define Tic_O     0b01111101
#define Tic_B     0b01111111
#define Tic_F     0b00001111
#define Tic_U     0b01110101
#define Tic_H     0b00110111
#define Tic_L     0b01000101
#define Tic_Y     0b01110110
#define Tic_I     0b00110000
#define Tic__     0b01000000
#define Tic_Poisk 0b00111101
#define Tic_Minus 0b00000010

#define Lcd_Comma Tic_Comma
#define Lcd_P     Tic_P
#define Lcd_S     Tic_S
#define Lcd_T     Tic_T
#define Lcd_E     Tic_E
#define Lcd_N     Tic_N
#define Lcd_G     Tic_G
#define Lcd_A     Tic_A
#define Lcd_B     Tic_B
#define Lcd_I     Tic_I
#define Lcd_F     Tic_F
#define Lcd_O     Tic_O
#define Lcd_U     Tic_U
#define Lcd_H     Tic_H
#define Lcd_L     Tic_L
#define Lcd_Y     Tic_Y
#define Lcd_c     Tic_c
#define Lcd_q     Tic_q
#define Lcd__     Tic__
#define Lcd_Poisk Tic_Poisk
#define Lcd_Minus Tic_Minus

#define Lcd4i     4
#define Lcd4f     2


	void BusOut(uint8_t Bus);

	void    cMEM(void);
	void    wMEM(void);
	uint8_t rMEM(void);

	void ScanerF(void);
	void ScanerM(void);
	void SetMute(void);

	void RsControl  (void);
	void UapyControl(uint8_t level);

	void VolumeUp  (void);
	void VolumeDown(void);

	void uart_putstr(char *p);

	void wBufEEPROM(uint16_t addr, uint8_t *buff, uint8_t size);
	void rBufEEPROM(uint16_t addr, uint8_t *buff, uint8_t size);

	void SQLup  (void);
	void SQLdown(void);

	void PDUsqlUp  (void);
	void PDUsqlDown(void);

	void PDUvolUp  (void);
	void PDUvolDown(void);

	void UapyUp  (void);
	void UapyDown(void);

	  uint8_t * ul2a(uint32_t c,uint8_t len);
	  uint32_t  a2l(uint8_t *p);

void (* RcCom[4])(void) =
     {
	PDUsqlUp,
	PDUsqlDown,
	PDUvolUp,
	PDUvolDown
     };

/////////////////////////////////////////////////
void SetFmax(void) { FrcvMax = RCV_MAX; return; }
void SetFmin(void) { FrcvMin = RCV_MIN; return; }

/////////////////
void KeyNop(void)
{
  return;
}

//////////////////////////
//////////////////////////
volatile uint8_t  BeepCnt;
volatile uint8_t  BeepTmr;

#define BEEP_Port_ini   DDRD
#define BEEP_Port      PORTD
#define BEEP_Bit         PD4

#define BEEP_T      25  // ms
#define BEEP_F     900  // Hz

#define BEEP_TMR  (7812/BEEP_F)
#define BEEP_LEN  (255-BEEP_T*15/BEEP_TMR)


//////////////////////////////////////////
void Beep(void) { BeepCnt = BEEP_LEN; return; }

/////////////////
void Beeper(void)
{
   if(++BeepTmr > BEEP_TMR)
   {
	BeepTmr = 0;

     if(BeepCnt)
     {
	if(++BeepCnt & 0x01)
	{
	   SetBit(BEEP_Port,BEEP_Bit);
	}
	else
	   ClrBit(BEEP_Port,BEEP_Bit);
     }
   }
   return;
}

///////////////////
////// U-apy //////
///////////////////

#define UapyNN  10
#define UapyHi  235
#define UapyLo  210

volatile uint8_t  Uapy = 0xff;
volatile uint8_t  UapyFlg = 1;
	 uint8_t  UapySet = 0xff;
	 uint8_t  UapyCnt;
	 uint16_t SmeterUapy = 0;

//////////////////
void UapyOut(void)
{
     OCR1B = Pwm2 = Uapy;
}

/////////////////////////////////////
void BusPause(void) { _delay_us(5); }

///////////////////
void BusStrob(void)
{
     BusPause();
	  SetBit(BUS_Port,BUS_BitClc);
     BusPause();
	  ClrBit(BUS_Port,BUS_BitClc);
}

////////////////////////
void BusOut(uint8_t Bus)
{
  uint8_t i;

    for(i=0; i < 8; i++)
    {

      if(Bus & 0x80)
	 SetBit(BUS_Port,BUS_BitDat);
      else
	 ClrBit(BUS_Port,BUS_BitDat);

      BusStrob();

      Bus <<= 1;
    }
    return;
}

volatile uint8_t  ExtPort = 0;

/////////////////////
void ExtPortSet(void)
{
     BusOut(ExtPort);

     BusPause();
	  SetBit(Ext_Port,Ext_Load);
     BusPause();
	  ClrBit(Ext_Port,Ext_Load);

     return;
}

///////////////////////////////////////////////////////////////////
void SetExtBit(uint8_t bit) { ExtPort |=  _BV(bit); ExtPortSet(); }
void ClrExtBit(uint8_t bit) { ExtPort &= ~_BV(bit); ExtPortSet(); }



///////////////////////////
uint8_t LcdDigit(uint8_t c)
{
  static char TicDigit[]={
	0b01111101,     // 0
	0b00110000,     // 1
	0b01011011,     // 2
	0b01111010,     // 3
	0b00110110,     // 4
	0b01101110,     // 5
	0b01101111,     // 6
	0b00111000,     // 7
	0b01111111,     // 8
	0b01111110,     // 9
	0b00111111,     // A
	0b01100111,     // B
	0b01001101,     // C
	0b01110011,     // D
	0b01001111,     // E
	0b00001111};    // F

	return(TicDigit[(c & 0x0F)]);
}

volatile uint8_t  LcdWork = 1;
volatile uint8_t  LcdBuf [LcdBufLEN];

volatile uint8_t  LcdYes;
volatile uint8_t  LcdCnt;
volatile uint8_t  LcdFlg;
volatile uint16_t LcdBlink;

/////////////////
void LcdTmr(void)
{
   if(++LcdCnt > 50)
   {
	LcdCnt = 0;
	LcdYes++;

     if(LcdBlink)
     {
	  LcdFlg = (LcdFlg)? 0:1;
     }
     else LcdFlg = 0;
   }
   return;
}


///////////////////
void TicLoad(void)
{
     BusPause();
	  SetBit(TIC_Port,TIC_Load);
     BusPause();
	  ClrBit(TIC_Port,TIC_Load);
}

/////////////////
void TicOut(void)
{
   int8_t i;
  uint8_t c;

  if(LcdYes)
  {
     LcdYes = 0;

     for(i=8; i >= 0; i--)
     {
       c = LcdBuf[i];

       if((LcdBlink >> i) & 0x0001)
       {
	   if(LcdFlg) c = 0;
       }

       if((i == 8) && (c & 0x80))
       {
	    BusOut(c | 0x40);
       }
       else BusOut(c & 0xBF);

       if(c & 0x40)
	  SetBit(BUS_Port,BUS_BitDat);
       else
	  ClrBit(BUS_Port,BUS_BitDat);

       BusStrob();
     }
     TicLoad();
  }
  return;
}


volatile uint8_t  KbdPos = 0;
volatile uint8_t  KbdKbd = 0;
volatile uint8_t  KbdYes = 0;
volatile uint8_t  KbdInd;
volatile uint8_t  KbdCnt;
volatile uint8_t  KbdOld;

volatile uint8_t  TmrInd;

/////////////////
void KbdWrk(void)
{
    if(KbdCnt < 128)
       KbdCnt++;

    BusOut(0x00);

    if( bit_is_clear(KBD_Port_Inp,KBD_BitInp1) ||
	bit_is_clear(KBD_Port_Inp,KBD_BitInp2) )
    {
       BusOut(~_BV(KbdPos));

       Nop();
       Nop();
       Nop();
       KbdInd = 255;

       if(bit_is_clear(KBD_Port_Inp,KBD_BitInp1)) KbdInd = KbdPos;
       if(bit_is_clear(KBD_Port_Inp,KBD_BitInp2)) KbdInd = KbdPos+8;

       if(KbdInd < 255)
       {
	 if(KbdYes && KbdOld == KbdInd)
	 {
	    if( KbdInd == IND_Pls ||
		KbdInd == IND_Mns )
	    {
	      if(KbdCnt > 50)
	      {
		 KbdKbd = Ind2KBD[KbdInd]+1;
	      }
	    }
	 }
	 else
	 {
	    KbdYes = 1;
	    KbdCnt = 0;
	    KbdOld = KbdInd;
	    KbdKbd = Ind2KBD[KbdInd]+1;
	 }
       }
    }
    else
    {
       KbdYes = 0;
    }

    if(++KbdPos >= 8) KbdPos = 0;
}

////////////////////
uint8_t GetKbd(void)
{
  uint8_t tmp;

      //cli();
      tmp = KbdKbd;

   if(tmp)
   {
      KbdKbd = 0;
   }
   //sei();
   return(tmp);
}

////////////////////
//     ENCODER    //
////////////////////

#define ENC_CONCT   20  // for encoder's good work

#define Enc_Port_ini    DDRC
#define Enc_Port_Inp    PINC
#define Enc_Port       PORTC
#define Enc_BitInp1      PC0
#define Enc_BitInp2      PC1

volatile uint8_t EnCoderYes;
volatile uint8_t EnCoderCnt;
volatile uint8_t EnCoderWii;
volatile uint8_t EnCoderInp = 0;

//////////////////
void EnCoder(void)
{
   if(EnCoderYes)
   {
       if(bit_is_clear(Enc_Port_Inp,Enc_BitInp1))
       {
	 if(++EnCoderWii > 2)
	 {
	   EnCoderYes = 0;
	   EnCoderCnt = 0;

	   if(bit_is_clear(Enc_Port_Inp,Enc_BitInp2))
	   {
	      KbdKbd = KBD_Pls+1;
	   }
	   else
	   {
	      KbdKbd = KBD_Mns+1;
	   }
	 }
       }
       else
	  EnCoderWii = 0;
   }
   else
   {
       if(bit_is_set(Enc_Port_Inp,Enc_BitInp2) &&
	  bit_is_set(Enc_Port_Inp,Enc_BitInp1) )
       {
	  if(EnCoderCnt > ENC_CONCT)
	  {
	     EnCoderYes = 1;
	  }
	  else
	     EnCoderCnt++;
       }
   }
   return;
}

///////////////////////////
//      RC-5 decoder     //
///////////////////////////

#define RC5_Port_ini   DDRD
#define RC5_Port_Inp   PIND
#define RC5_Port      PORTD
#define RC5_BitInp      PD2

volatile uint8_t RC5flg;
volatile uint8_t RC5err;
volatile uint8_t RC5num;
volatile uint8_t RC5cnt;
volatile uint8_t RC5wrk;
volatile uint8_t RC5prg;

volatile uint16_t RC5inp;
volatile uint16_t RC5old;
volatile uint16_t RC5rcv;
volatile uint16_t RC5tmp;

volatile  int8_t RC5cnt0a;
volatile  int8_t RC5cnt1a;
volatile  int8_t RC5cnt0b;
volatile  int8_t RC5cnt1b;

/////////////////////////
//  Interrupt by INT0  //
/////////////////////////
SIGNAL (SIG_INTERRUPT0)
{
   if(!RC5flg)
   {
       RC5flg++;
       RC5cnt = 0;
       RC5rcv = 0;
       RC5num = 0;
       RC5err = 0;
   }
   return;
}

////////////////////
void RC5AskBit(void)
{
  uint8_t a;
  uint8_t b;

     if(++RC5num < 14)
     {
     if(abs(RC5cnt1a-RC5cnt0a) < 2) RC5err++;
     if(abs(RC5cnt1b-RC5cnt0b) < 2) RC5err++;

       a = (RC5cnt1a > RC5cnt0a)? 1:0;
       b = (RC5cnt1b > RC5cnt0b)? 1:0;

       if(a == b) RC5err++;

	RC5rcv <<= 1;
	RC5rcv  |= (a > b)? 1:0;
     }
     else
     {
	if(RC5num > 14)
	{
	   if(!RC5err && RC5rcv & 0x1000)
	   {
	       RC5inp =  RC5rcv;
	   }
	   RC5flg = 0;
	}
     }
     RC5cnt1a =
     RC5cnt0a =
     RC5cnt1b =
     RC5cnt0b = 0;
     return;
}

/////////////////////////////
uint8_t RC5corrector(uint8_t i)
{
  static char corrector[12]={26,26,25,26,
			     26,26,25,26,
			     26,26,25,26};

  return((i > 11)? 26:corrector[i]);
}

//////////////////////
void RC5receiver(void)
{
     if(RC5flg)
     {
	if(RC5cnt > 13)
	{
	   if(bit_is_clear(RC5_Port_Inp,RC5_BitInp))
	   {
		RC5cnt0b++;
	   }
	   else
		RC5cnt1b++;
	}
	else
	{
	   if(bit_is_clear(RC5_Port_Inp,RC5_BitInp))
	   {
		RC5cnt0a++;
	   }
	   else
		RC5cnt1a++;
	}
	if(RC5cnt++ > RC5corrector(RC5num))
	{
	   RC5cnt = 0;
	   RC5AskBit();
	}
     }
     return;
}

////////////////////////
void RC5write(uint8_t c)
{
     if(c < 18)
     {
       if(RC5prg > 3)
       {
	  RC5prg = 0;

	  wBufEEPROM ((uint16_t)(c<<2) + ADR_EE_RC5, (uint8_t *)&RC5old, 2);
	  uart_putstr("\nOK");
       }
     }
     return;
}

////////////////////////////
uint16_t RC5_Kbd(void)
{
  uint8_t i;

     for(i = 0; i < 18; i++)
     {
       rBufEEPROM((uint16_t)(i<<2) + ADR_EE_RC5, (uint8_t *)&RC5tmp, 2);

       if( RC5tmp == (RC5inp & 0x07ff) )
       {
	 if(RC5old != RC5inp || i >= KBD_Pls)
	 {
	    if(i < 14)
	    {
	       Beep();
	       RC5old = RC5inp;  return i+1;
	    }
	    else
	    {
	      (* RcCom[i-14])(); return 0;
	    }
	 }
       }
     }
     return 0;
}


/////////////////////////////
/////////////////////////////
     volatile  int8_t TMRxxx;

void TmrYYY(void)
{
  if(TMRxxx)
  {
     TMRxxx = 0;

     KbdWrk();
     TicOut();
  }
  return;
}

/////////////////////////
// Interrupt by Timer0 //
/////////////////////////
//
// Start by 64 mks
//
SIGNAL (SIG_OVERFLOW0)
{
  RC5receiver();

  Beeper();

  EnCoder();

  if(TmrInd <  254)
     TmrInd++;

  if(TmrInd ==  64)
  {
     TmrInd = 0;
     TMRxxx++;

     LcdTmr();

     if(MuteCnt < MUTE)
	MuteCnt++;

     if(++TimeTmr == 0)   // 1048 ms
     {
       if(TimeClr)
       {
	  if(--TimeClr ==  0) AutoClr++;
       }
     }
  }
}

////////////////////////////
// Interrupt by Rx RS-232 //
////////////////////////////
SIGNAL (SIG_UART_RECV)
{
  uint8_t tmp;

  if(RxInd < RxInpBufLEN)
  {
     RxInpBuf[RxInd++] = UDR;
  }
  else tmp = UDR;
}

//--------------//
void ioinit (void)
{
    // Init PORTs

    Port4Out(BUS_Port_ini,BUS_BitClc); // Bus clc -> Out
    Port4Out(BUS_Port_ini,BUS_BitDat); // Bus dat -> Out

    Port4Out(Ext_Port_ini,Ext_Load);   // Port Extention 74HC595 load -> Out

    Port4Inp(KBD_Port_ini,KBD_BitInp1); // KBD inp1 -> Input
    Port4Inp(KBD_Port_ini,KBD_BitInp2); // KBD inp2 -> Input
    SetBit  (KBD_Port,KBD_BitInp1);     //     Input (+Ucc --> "1")
    SetBit  (KBD_Port,KBD_BitInp2);     //     Input (+Ucc --> "1")

    Port4Out(I2C_Port_ini,I2C_BitClc); // I2C clc -> Out
    Port4Out(I2C_Port_ini,I2C_BitDat); // I2C dat -> Out

    SetBit(I2C_Port,I2C_BitClc);  // Init I2C
    SetBit(I2C_Port,I2C_BitDat);

    // BEEPER

    Port4Out(BEEP_Port_ini,BEEP_Bit);  // Beeper -> Out

    // ENCODER

    Port4Inp(Enc_Port_ini,Enc_BitInp1); // EnCoder inp1 -> Input
    Port4Inp(Enc_Port_ini,Enc_BitInp2); // EnCoder inp2 -> Input
    SetBit  (Enc_Port,Enc_BitInp1);     //     Input (+Ucc --> "1")
    SetBit  (Enc_Port,Enc_BitInp2);     //     Input (+Ucc --> "1")

    // TIC-9162

    Port4Out(TIC_Port_ini,TIC_Load);   // Tic9162 load -> Out


    // Init ADC

    Port4Inp(DDRC,PC0);
    Port4Inp(DDRC,PC1);
    Port4Inp(DDRC,PC2);
    Port4Inp(DDRC,PC3);

    ADMUX  = 0x00;
    ADCSRA = 0x85; // 1000 0101  ADC on, interrupt disable, MCU clock / 32

    // Init Timer0

    TCCR0  = 0x01;       // Timer0 running on MCU clock / 1
    SetBit(TIMSK,TOIE0); // enable interrupt from Timer0


    // Init Timer1

    Port4Out(DDRB,PB1); // output PWM1 (PB1/OC1A)
    Port4Out(DDRB,PB2); // output PWM2 (PB2/OC1B)

    TCCR1A = 0B10100001; // Timer1 is duble 8-bit PWM
    TCCR1B = 0B00000001; // Timer1 running on MCU clock / 1

    OCR1A = Pwm1 = 255;  // set PWM1 value
    OCR1B = Pwm2 = 255;  // set PWM2 value

    // Init INT0

    Port4Inp(RC5_Port_ini,RC5_BitInp); // RC5 inp -> Input
    SetBit  (RC5_Port,    RC5_BitInp); // RC5 Input (+Ucc --> "1")
		       //         _
    MCUCR = 0x03;      // int0  _|   (rising edge)

    SetBit(GICR,INT0); // enable interrupt from INT 0

    // ### RS-232 ###

    Port4Inp(DDRD,PD0); // RS-232 Rx (Inp)
    Port4Out(DDRD,PD1); // RS-232 Tx (Out)

    UBRRL = (F_CPU / (16 * 9600UL)) - 1;        // Bd 9600
    UCSRB = _BV(TXEN) | _BV(RXEN) | _BV(RXCIE); // Tx,Rx,Rx interrupt enable

    // enable interrupts
       sei ();
       return;
}

///////////////////////////////////////
void Pause200(void) { _delay_us(200); }

/////////////////////
void sleep(int16_t s)
{
  for(s*=5; s > 0; s--)
  {
  TmrYYY();
      Pause200();
  }
}


////////////////////////
int uart_putchar(char c)
{
  if (c == '\n')
     uart_putchar('\r');

  //loop_until_bit_is_set(UCSRA, UDRE);

  while(bit_is_clear(UCSRA, UDRE))
  {
    TmrYYY();
  }
  UDR  = c;
  return 0;
}

////////////////////////
void uart_putstr(char *p)
{
   while(*p)
   {
     uart_putchar(*p++);
   }
   return;
}

////////////////////////

void PutHEX(uint8_t c)
{
  static char hex[16]= {'0','1','2','3','4','5','6','7',
			'8','9','A','B','C','D','E','F'};

 uart_putchar('[');

   uart_putchar(hex[c >>   4]);
   uart_putchar(hex[c & 0x0f]);

 uart_putchar(']');
   return;
}

//////////////////
void Rcv2Lcd(void)
{
  char *p;

  uint8_t i;
  uint8_t flag = 0;

    p = ul2a(FrcvWork,6);

    for(i=0; i < 6; i++)
    {
      cccBuf[i]=*p;

      if(*p != '0') flag++;

	       LcdBuf[i+Lcd4f] &= Lcd_Comma;
      if(flag) LcdBuf[i+Lcd4f] |= LcdDigit(*p-'0');

      p++;
    }
    return;
}


////////////////////////////////////
void Int2Lcd(uint16_t c, uint8_t sw)
{
  char *p;

  uint8_t i;
  uint8_t flag = 0;

    p = ul2a(c,4);

    for(i=0; i < 4; i++)
    {
      if(*p != '0' || i == 3) flag++;

	       LcdBuf[i+sw] &= Lcd_Comma;
      if(flag) LcdBuf[i+sw] |= LcdDigit(*p-'0');

      p++;
    }
    return;
}


//////////////////
void Ind2Lcd(void)
{
  uint8_t *p;
  uint8_t  i;

    p = ul2a(IndMEM,2);

    for(i=0; i < 2; i++)
    {
       LcdBuf[i] &= Lcd_Comma;
       LcdBuf[i] |= LcdDigit(*p-'0');
       p++;
    }
  //LcdBuf[2] &= Lcd_Comma;
  //LcdBuf[2] |= Lcd_Minus;
    return;
}

#define CH_POROG  0x42
#define CH_SMETR  0x43
#define CH_GND    0x4f

#define CH_VOLUME  0x40
#define CH_JOYSTIC 0x41


//--------------//
uint16_t GetADC (uint8_t chanel)
{
  ADMUX = chanel;

  Pause200();
  SetBit(ADCSRA,ADSC); // start ADC

  while(bit_is_set(ADCSRA,ADSC))
  {
    TmrYYY();
    Nop();
  }

  Pause200();
  SetBit(ADCSRA,ADSC); // start ADC

  while(bit_is_set(ADCSRA,ADSC))
  {
    TmrYYY();
    Nop();
  }
  return(ADCW);
}

/////////////////////////////
void SmeterLcd(uint8_t level)
{
   int8_t i;
  uint8_t f = 0;

     for(i = 7; i >= 0; i--)
     {
	if(level & 0x80) f++;

	      LcdBuf[i] &= ~Lcd_Comma;
	if(f) LcdBuf[i] |=  Lcd_Comma;

	level <<= 1;
     }
     return;
}

////////////////////
void SmeterPrn(void)
{
  uint8_t *p;
  uint8_t  i;

  uint16_t level = LoBYTE(GetADC(CH_SMETR)>>2);

    SmeterLcd(level);

    uart_putchar(' ');

    p = ul2a(level,4);

    for(i=0; i < 4; i++)
    {
	uart_putchar(*p++);
    }
    return;
}


////////////
void wEEPROM(uint16_t addr, uint8_t byte)
{
     while(bit_is_set(EECR,EEWE))
     {
       Nop();
     }
     EEAR = addr;
     EEDR = byte;
     cli ();
	 SetBit(EECR,EEMWE);
	 SetBit(EECR,EEWE ); // start write to eeprom
     sei ();
     return;
}

///////////////
uint8_t rEEPROM(uint16_t addr)
{
     while(bit_is_set(EECR,EEWE))
     {
       Nop();
     }
     EEAR = addr;
     SetBit(EECR,EERE);
     return(EEDR);
}

///////////////
void wBufEEPROM(uint16_t addr, uint8_t *buff, uint8_t size)
{
     uint8_t i;

     for(i=0; i < size; i++)
     {
       wEEPROM(addr++,*buff); buff++;
     }
     return;
}

///////////////
void rBufEEPROM(uint16_t addr, uint8_t *buff, uint8_t size)
{
     uint8_t i;

     for(i=0; i < size; i++)
     {
	*buff = rEEPROM(addr++);
	 buff++;
     }
     return;
}

/////////////////////////
void SetFwork(uint32_t f)
{
     if( f <=  FrcvMax &&
	 f >=  FrcvMin )
     {
	 FrcvWork = (f/5)*5;
     }
     KeyRADIO();
}

/////////////////////////
void Lm7001Out(uint8_t c)
{
  uint8_t i;

    for(i=0; i < 8; i++)
    {
      if(c & 0x01)
	 SetExtBit(LM7001_BitDat);
      else
	 ClrExtBit(LM7001_BitDat);

	BusPause();
	 SetExtBit(LM7001_BitClc);
	BusPause();
	 ClrExtBit(LM7001_BitClc);

      c >>= 1;
    }
    return;
}

////////////////////
void Lm7001Set(void)
{
  static uint8_t  Mod4Lm7001[3] = {0xE6,0xE4,0xE2};
	 uint16_t fPCH2;

     FrcvPCH2 = (RCVPCH1-RCVPCH2)-(FrcvWork-((FrcvWork/50)*50));
	fPCH2 = FrcvPCH2/5;

     cli ();
     SetExtBit(LM7001_BitCEN);

     Lm7001Out(LoBYTE(fPCH2));
     Lm7001Out(HiBYTE(fPCH2));

     flagBo = Mod4Lm7001[Modulation];

	  ///////////////////////////////////////
	  //
	  //  S R2 R1 R0 TB Bo3 Bo2 Bo1
	  //
	  //  R0=0, R1=1, R2=1 --> 5kHz
	  //  R0=1, R1=1, R2=0 --> 1kHz
	  //   S=1  --> FMIN
	  //   S=0  --> AMIN
	  //
	  //      flagBo = 0xE0;  // 5kHz, FMIN
	  //      flagBo = 0xB0;  // 1kHz, FMIN  (Max 16383mHz)
	  //
	  // Bo1  --> 1 - Mute ON 0 - Mute OFF
	  // Bo2  --> 1 - WFM  0 - NFM
	  // Bo3  --> 1 - FM   0 -  AM

     if(FlagMute) { flagBo |= 0x01; }

     Lm7001Out(flagBo);

     ClrExtBit(LM7001_BitCEN);
     ClrExtBit(LM7001_BitDat);
     sei ();
     return;
}

//////////////////////////////////////
void I2CPause(void) { _delay_us(10); }

///////////////////
void I2CStart(void)
{
     SetBit(I2C_Port,I2C_BitClc);
     SetBit(I2C_Port,I2C_BitDat);

    I2CPause();
     ClrBit(I2C_Port,I2C_BitDat);

    I2CPause();
     ClrBit(I2C_Port,I2C_BitClc);
}

//////////////////
void I2CStop(void)
{
     ClrBit(I2C_Port,I2C_BitDat);
     SetBit(I2C_Port,I2C_BitClc);

    I2CPause();
     SetBit(I2C_Port,I2C_BitDat);
}

//////////////////
void I2CSync(void)
{
    I2CPause();
     SetBit(I2C_Port,I2C_BitClc);

    I2CPause();
     ClrBit(I2C_Port,I2C_BitClc);
}

//////////////////////////
void I2COutByte(uint8_t c)
{
  uint8_t i;

    for(i=0; i < 8; i++)
    {

      if(c & 0x80)
	 SetBit(I2C_Port,I2C_BitDat);
      else
	 ClrBit(I2C_Port,I2C_BitDat);

	I2CSync();

	c <<= 1;
    }
    I2CSync(); // ASK
    return;
}

////////////////////
void KSH14xPLL(void)
{
  uint32_t fPCH1;
  uint8_t   Band;


     Band = CONST__LoBAND;

     if(FrcvWork > 155000L) Band = CONST__MiBAND;
     if(FrcvWork > 440000L) Band = CONST__HiBAND;


     fPCH1 = (uint16_t)((FrcvWork + RCVPCH1)/50);

     I2CStart();

     I2COutByte(0b11000000);   // Tuner Address
     I2COutByte(HiBYTE(fPCH1));
     I2COutByte(LoBYTE(fPCH1));
     I2COutByte((TunerGain)? CONST__GAINon:CONST__GAINoff);
     I2COutByte(Band);

     I2CStop();
     return;
}

//////////////////
void fWorkPrn(void)
{
  uint8_t i;

     Rcv2Lcd(); sleep(5);

     uart_putstr("\nF ");

     GetADC(CH_SMETR);

     for(i = 0; i < 6; i++)
     {
	uart_putchar(cccBuf[i]);

	if(i == 2)
	      uart_putchar('.');
     }
     SmeterPrn();
     SmeterPrn();
     SmeterPrn();
     return;
}

//////////////////
void SetFrcv(void)
{
     Lm7001Set();
     KSH14xPLL();

     fWorkPrn();
     return;
}


// --------- //
int main (void)
{
  uint8_t key;

    ioinit ();

    SetFmax();
    SetFmin();

    rBufEEPROM (ADR_EE_vMax, (uint8_t *)&Vol_Max, 1);
    rBufEEPROM (ADR_EE_vMin, (uint8_t *)&Vol_Min, 1);

    rBufEEPROM (ADR_EE_vWrk, (uint8_t *)&Pwm1, 1);

    if(Vol_Max == 0xff)
    {
       Vol_Max = VolumeMAX;
       Vol_Min = VolumeMIN;
    }
    if(Pwm1 == 0xff)
    {
       Pwm1 = Vol_Max;
    }
    OCR1A = Pwm1;

    FlagMute = 1;
    Modulation = WFM;
    FrcvWork   = 106600;
    rMEM();              // IndMEM = 0

    KeyRADIO(); sleep(1000);
    RC5wrk = 1;

    for (;;)
    {
      TmrYYY();

      RsControl();

      if(AutoClr)
      {
	 AutoClr = 0; KeyRADIO();
      }

      key = GetKbd();

      if(RC5inp)
      {
	if(RC5wrk)
	{
	   key = RC5_Kbd();
	}
	else
	{
	   uart_putstr("RC5:");
	   PutHEX(HiBYTE(RC5inp));
	   PutHEX(LoBYTE(RC5inp));
	   uart_putstr(PROMPT);

	   RC5inp &= 0x07ff;

	   if(RC5inp == RC5old)
	   {
	      RC5prg++;
	   }
	   else
	   {
	      RC5prg = 0;
	      RC5old = RC5inp;
	   }
	}
	RC5inp=0;
      }

      /*
      if(EnCoderInp)
      {
	 uart_putchar(EnCoderInp);
	 EnCoderInp = 0;
      }
      */

      if(key--)
      {
	Beep();
	(* SwKey[((key) & 0x0f) | ((Regim == FINPUT)? 0x10:0x00)] )();

	LcdYes++;
      }

      if(FlagScanF) { ScanerF(); LcdYes++; }
      if(FlagScanM) { ScanerM(); LcdYes++; }

      SmeterLcd(Smeter = LoBYTE(GetADC(CH_SMETR)>>2));

      SetMute();

      UapyControl(Smeter);
    }
}

//////////////////
void FrcvUpDown(void)
{
    switch(direction)
    {
      case UP:   if(FrcvWork <  FrcvMax)
		 {
		    FrcvWork += FrcvStep;
		 }
		 else
		    direction = DOWN;

		 break;

      case DOWN: if(FrcvWork >  FrcvMin)
		 {
		    FrcvWork -= FrcvStep;
		 }
		 else
		    direction = UP;

		 break;
    }
    return;
}


//////////////////
void KeyPlus(void)
{
   if(Regim == RADIO)
   {
     direction = UP;

     FrcvUpDown();
     SetFrcv();
   }

   if(Regim == WR_MEM || Regim == RC_MEM)
   {
      if(++IndMEM > NUM_MEM-1)
	   IndMEM = 0;
      Ind2Lcd();

      if(Regim == RC_MEM) rMEM();
   }

   if(Regim == SCAN_M && FlagBank == 0)
   {
      if(++ScanMBank > 3)
	   ScanMBank = 0;

      Int2Lcd(ScanMBank,Lcd4i);
   }

   if(Regim == SET_SQL) SQLup();
   if(Regim == SET_VOL) VolumeUp();
   if(Regim == UAPY   ) UapyUp();

   return;
}

//////////////////
void KeyMinus(void)
{
   if(Regim == RADIO)
   {
     direction = DOWN;

     FrcvUpDown();
     SetFrcv();
   }

   if(Regim == WR_MEM || Regim == RC_MEM)
   {
      if(--IndMEM < 0)
	   IndMEM = NUM_MEM-1;
      Ind2Lcd();

      if(Regim == RC_MEM) rMEM();
   }

   if(Regim == SCAN_M && FlagBank == 0)
   {
      if(--ScanMBank < 0)
	   ScanMBank = 3;

      Int2Lcd(ScanMBank,Lcd4i);
   }

   if(Regim == SET_SQL) SQLdown();
   if(Regim == SET_VOL) VolumeDown();
   if(Regim == UAPY   ) UapyDown();

   return;
}


///////////////////////////////////////////
static uint8_t On2Lcd[5] = {    0,Lcd_O,Lcd_N};
static uint8_t Of2Lcd[5] = {Lcd_O,Lcd_F,Lcd_F};

/////////////////////////
void Flg2Lcd( uint8_t sw)
{
     memcpy((uint8_t *)&LcdBuf[5],(sw)? On2Lcd:Of2Lcd,3);
     return;
}

/////////////////////////////////////////////////////////
static uint8_t Gain2Lcd[5] = {Lcd_G,Lcd_A,Lcd_I,Lcd_N,0};
static uint8_t Uapy2Lcd[5] = {Lcd_U,Lcd_A,Lcd_P,Lcd_Y,0};
static uint8_t Banc2Lcd[5] = {Lcd_B,Lcd_A,Lcd_N,Lcd_c,0};
static uint8_t Step2Lcd[5] = {Lcd_S,Lcd_T,Lcd_E,Lcd_P,0};
static uint8_t _sql2Lcd[5] = {Lcd_S,Lcd_q,Lcd_L,0,0};
static uint8_t _vol2Lcd[5] = {Lcd_U,Lcd_O,Lcd_L,0,0};


#define sGain 0
#define sUapy 1
#define sBanc 2
#define sStep 3
#define sSql  4
#define sVol  5

void Str2Lcd(uint8_t sw)
{
  uint8_t *p;

	if(sw == sGain) p = Gain2Lcd;
   else if(sw == sUapy) p = Uapy2Lcd;
   else if(sw == sBanc) p = Banc2Lcd;
   else if(sw == sStep) p = Step2Lcd;
   else if(sw == sSql ) p = _sql2Lcd;
   else                 p = _vol2Lcd;

   memcpy((uint8_t *)LcdBuf,p,5);
}

//////////////////
void KeySTEP(void)
{
     switch(Regim)
     {
       case STEP:  if(++FindStep >= 8)
			FindStep  = 0;
		   FrcvStep = Step[FindStep];
       case RADIO:
		   Str2Lcd(sStep);
		   Int2Lcd(FrcvStep,Lcd4i);

		   Regim = STEP;
     }
     return;
}

//////////////////
void Mod2Lcd(void)
{
  static uint8_t Mod4Lcd[3] = {0b01100010,0b01000011,0b01100011};

  LcdBuf[1] = Mod4Lcd[Modulation];
}

//////////////////
void KeyMOD(void)
{
   if(Regim == RADIO)
   {
     if(++Modulation > AM)
	  Modulation = WFM;

     Mod2Lcd();
     Lm7001Set();
   }
   return;
}

//////////////////
void KeyRADIO(void)
{
     if(Regim == SET_VOL)
     {
	wBufEEPROM (ADR_EE_vWrk, (uint8_t *)&Pwm1, 1);
     }

     if(Regim == SCAN_F)
     {
	FlagScanF = 0;
	SetFmax();
	SetFmin();
     }
     FlagScanM = 0;
     Regim = RADIO;

     TimeClr = 0;

     LcdBuf[0] = Lcd_P;
   //LcdBuf[2] = 0;
     LcdBlink  = 0;

     Mod2Lcd();
     SetFrcv();
}


//////////////////
void KeyGAIN(void)
{
     switch(Regim)
     {
       case GAIN:  TunerGain = (TunerGain)? 0:1;
		   KSH14xPLL();

       case RADIO:
		   Str2Lcd(sGain);
		   Flg2Lcd(TunerGain);

		   Regim = GAIN;
     }
     return;
}

/////////////////
void UapyUp(void)
{
     if(UapyFlg == 2)
     {
	if(Uapy < 255)
	   Uapy++;

	UapyOut();
	Int2Lcd(Uapy,Lcd4i);
     }
     return;
}
///////////////////
void UapyDown(void)
{
     if(UapyFlg == 2)
     {
	if(Uapy)
	   Uapy--;

	UapyOut();
	Int2Lcd(Uapy,Lcd4i);
     }
     return;
}

//////////////////
void KeyUapy(void)
{
     switch(Regim)
     {
       case UAPY:
			if(UapyFlg == 1)
			{
			   UapyFlg = 2;
			   Uapy = UapySet;
			}
		   else if(UapyFlg == 2)
			{
			   UapyFlg = 0;
			   UapySet = Uapy;
			   Uapy = 0xff;
			}
		   else {
			   UapyFlg = 1;
			}
			UapyOut();

       case RADIO:
		   Str2Lcd(sUapy);

		   if(UapyFlg != 2)
		   {
		      Flg2Lcd(UapyFlg);
		   }
		   else
		   {
		      Int2Lcd(Uapy,Lcd4i);
		   }
		   Regim = UAPY;
     }
     return;
}

//////////////////
void KeyFinp(void)
{
     switch(Regim)
     {
       case FINPUT:
       case  RADIO:
		    Mod2Lcd();

		    memset((void *)&LcdBuf[Lcd4f],0,6);

		    LcdBlink  = 0x0004;
		    LcdBuf[    0] = Lcd_F;
		    LcdBuf[Lcd4f] = Lcd__;

		    InpInd = 0;
		    Regim  = FINPUT;
		    break;

       case  WR_MEM:
		    cMEM();
		    LcdBlink = 0x01FF;
		    sleep(2000);
		    LcdBlink = 0x0003;
     }
     return;
}

////////////////////////
void Key_ii( uint8_t c )
{
  uint32_t f;

	LcdBlink <<= 1;

	LcdBuf[InpInd+Lcd4f] &= Lcd_Comma;
	LcdBuf[InpInd+Lcd4f] |= LcdDigit(c);;

	InpBuf[InpInd++] = c +'0';
	InpBuf[InpInd  ] = 0;

     if(InpInd >= InpBufLEN-1)
     {
	f = a2l((uint8_t *)InpBuf);

	SetFwork(f);
     }
     else
	LcdBuf[InpInd+Lcd4f] = Lcd__;
}

////////////////
void Key_0(void) { Key_ii(0); return; }
void Key_1(void) { Key_ii(1); return; }
void Key_2(void) { Key_ii(2); return; }
void Key_3(void) { Key_ii(3); return; }
void Key_4(void) { Key_ii(4); return; }
void Key_5(void) { Key_ii(5); return; }
void Key_6(void) { Key_ii(6); return; }
void Key_7(void) { Key_ii(7); return; }
void Key_8(void) { Key_ii(8); return; }
void Key_9(void) { Key_ii(9); return; }



///////////////////////////////////////
void KeySetHL(uint16_t addr, uint8_t c)
{
     switch(Regim)
     {
       case WR_H_L:
		    wBufEEPROM (addr, (uint8_t *)&FrcvWork, 4);

		    LcdBlink  = 0;
		    LcdBuf[0] = c;

		    sleep(2000);
		    KeyRADIO();
		    break;
       case  RADIO:
		    LcdBlink  = 0x0001;
		    LcdBuf[0] = c;
		    LcdBuf[1] = 0;

		    Regim  = WR_H_L;
     }
     return;
}

///////////////////
void KeySetHi(void) { KeySetHL(ADR_EE_Hi,Lcd_H); return; }
void KeySetLo(void) { KeySetHL(ADR_EE_Lo,Lcd_L); return; }

/////////////////////////////////////
uint8_t buff[8]; // for min size code
/////////////////////////////////////

////////////////
void wMEM(void)
{
     memcpy(buff,(uint8_t *)&FrcvWork,4);

     buff[3] = Modulation | 0xFC;

     wBufEEPROM (((uint16_t)IndMEM)<<2, buff, 4);
}

//////////////////
uint8_t rMEM(void)
{
     rBufEEPROM (((uint16_t)IndMEM)<<2, buff, 4);

     if((buff[3] & 0x03) != 0x03)
     {
	Modulation = buff[3] & 0x03;

	memcpy((uint8_t *)&FrcvWork,buff,3);
	SetFrcv();
     }
     else
     {
	 memset((void *)&LcdBuf[Lcd4f],Lcd_Minus,6);
	 return 1;
     }
     return 0;
}

////////////////
void cMEM(void)
{
     buff[0] =
     buff[1] =
     buff[2] =
     buff[3] = 0xff;

     wBufEEPROM (((uint16_t)IndMEM)<<2, buff, 4);
}

///////////////////
void KeyWrMem(void)
{
     switch(Regim)
     {
       case WR_MEM:
		    wMEM();
		    LcdBlink = 0;

		    sleep(2000);
		    KeyRADIO();
		    break;
       case  RADIO:
		    LcdBlink = 0x0003;
		    Ind2Lcd();

		    Regim  = WR_MEM;
     }
     return;
}

///////////////////
void KeyRcMem(void)
{
     if(Regim == RADIO)
     {
	Regim = RC_MEM;

	Ind2Lcd();
	rMEM();
     }
     return;
}

///////////////////
void KeyScanF(void)
{
     if(Regim == SCAN_F)
     {
	FlagScanF = (FlagScanF)? 0:1;
     }

     if(Regim == RADIO)
     {
	Regim = SCAN_F;

	LcdBuf[0] = Lcd_Poisk;  // "POISK"
	Mod2Lcd();

	rBufEEPROM (ADR_EE_Hi, (uint8_t *)&FrcvMax, 4);
	rBufEEPROM (ADR_EE_Lo, (uint8_t *)&FrcvMin, 4);

	if(FrcvMax < FrcvMin)
	{
	  rBufEEPROM (ADR_EE_Lo, (uint8_t *)&FrcvMax, 4);
	  rBufEEPROM (ADR_EE_Hi, (uint8_t *)&FrcvMin, 4);
	}
	if(FrcvMax & 0x80000000) SetFmax();
	if(FrcvMin & 0x80000000) SetFmin();

	if( FrcvWork > FrcvMax ||
	    FrcvWork < FrcvMin || FlagBegin)
	{
	    FrcvWork = FrcvMin;
	    FlagBegin= 0;
	    SetFrcv();
	}
	FlagScanF = 1;
     }
     return;
}

//////////////////
void ScanerF(void)
{
     FrcvUpDown(); SetFrcv();
}

///////////////////
void KeyScanM(void)
{
     if(Regim == SCAN_M)
     {
	if(FlagBank)
	{
	   FlagScanM = (FlagScanM)? 0:1;
	}
	else
	{
	   FlagScanM = 1;
	   FlagBank  = 1;
	}
     }

     if(Regim == RADIO)
     {
	Regim = SCAN_M;

	Str2Lcd(sBanc);
	Int2Lcd(ScanMBank,Lcd4i);
	FlagBank = 0;
     }
     return;
}

//////////////////
void ScanerM(void)
{
  static uint8_t ScanMmin[4] = { 1,25,50,75};
  static uint8_t ScanMmax[4] = {24,49,74,99};
  uint8_t IndTmp;

     if( IndMEM < ScanMmin[ScanMBank] ||
	 IndMEM > ScanMmax[ScanMBank] )
	 IndMEM = ScanMmin[ScanMBank];

     IndTmp = IndMEM;

     do
     {
	if(++IndMEM > ScanMmax[ScanMBank])
	{
	     IndMEM = ScanMmin[ScanMBank];
	}

	if(!rMEM())
	{
	   SmeterPrn();
	   SmeterPrn();
	   SmeterPrn();
	   SmeterPrn();
	   SmeterPrn();
	   SmeterPrn();
	   break;
	}

     } while(IndTmp != IndMEM);

     Ind2Lcd();
     return;
}


  /////////////////////
  /////////////////////
  volatile uint8_t SQL;

////////////////
void SQLup(void)
{
     if(SQL < 255)
	SQL++;

     Int2Lcd(SQL,Lcd4i);  TimeClr = AUTO_CLR;
}

//////////////////
void SQLdown(void)
{
     if(SQL)
	SQL--;

     Int2Lcd(SQL,Lcd4i);  TimeClr = AUTO_CLR;
}

///////////////////
void KeySQL(void)
{
     if(Regim == RADIO)
     {
	Regim = SET_SQL;

	Str2Lcd(sSql);
	Int2Lcd(SQL,Lcd4i);  TimeClr = AUTO_CLR;
     }
     return;
}

///////////////////
void PduRADIO(void)
{
   if( Regim != SET_VOL &&
       Regim != SET_SQL &&
       Regim != RADIO   )
   {
       KeyRADIO();
   }
   return;
}

//////////////////////////////////////////////////////////////////
void PDUsqlUp  (void) { PduRADIO(); KeySQL(); SQLup();   return; }
void PDUsqlDown(void) { PduRADIO(); KeySQL(); SQLdown(); return; }

//////////////////
void SetMute(void)
{
     if(FlagSQL == 0)
	return;

      if(Smeter > SQL || SQL == 0)
      {
      // Sound ON

	 FlagScanF = 0;
	 FlagScanM = 0;
	 MuteCnt = 0;

	 if(FlagMute)
	 {
	    FlagMute = 0;
	    Lm7001Set();
	 }
      }
      else
      {
      // Sound Off

	 if(MuteCnt >= MUTE)
	 {
	    if(!FlagMute)
	    {
		FlagMute = 1;
		Lm7001Set();
	    }
	 }
      }
      return;
}

void RsControl(void)
{
  uint8_t inp;
  uint8_t   i;
  uint32_t  f;

  if(Regim == RADIO || Regim == SCAN_F)
  {
     if(RxInd != TxInd)
     {
	if((inp = RxInpBuf[TxInd++]) != '\n' && EchoYes)
	{
	   uart_putchar(inp);
	}
	//uart_putchar(inp = RxInpBuf[TxInd++]);

	if(RxInd >= RxInpBufLEN)
	{
	   RcInd = 0;
	   RxInd = 0;
	   TxInd = 0;
	   uart_putstr(ERROR);

	   return;
	}
	if(isdigit(inp))
	{
	   RcInpBuf[RcInd++] = inp;
	}

	if(inp == '\r')
	{
	   RcInpBuf[RcInd] = '\0';
	   RcInd = 0;

	   if(Regim == SCAN_F)
	   {
	      RxInpBuf[0] = 0;
	      KeyRADIO();
	   }
	   f = a2l((uint8_t *)RcInpBuf);

	   i = (uint8_t)(f & 0x000000FF);

	   // use uint8_t inp like tmp

	   inp = (RxInpBuf[0] == '\n')? 1:0;

	   switch(RxInpBuf[inp])
	   {
	    case '+': KeyPlus (); break;
	    case '-': KeyMinus(); break;

	    case 'M':
		      if((Modulation = i & 0x03) == 3)
			  Modulation = 0;
		      Mod2Lcd();
		      Lm7001Set();
		      break;

	    case 'G': TunerGain = i & 0x01;
		      KSH14xPLL();
		      break;

	    case 'S': FindStep = i &0x07;
		      FrcvStep = Step[FindStep];
		      break;

	    case 'U': Uapy    = i;
		      UapyFlg = 2; UapyOut();
		      break;

	    case 'E': EchoYes = i;
		      break;

	    case 'L':
		      if(i == 2)
		      {
			 FlagSQL = 1;
		      }
		      else
		      {
			 FlagMute = i & 0x01;
			 FlagSQL = 0;
			 Lm7001Set();
		      }
		      break;

	    case 'R': RC5wrk = i & 0x01;
		      break;

	    case 'W': RC5write(i);
		      break;

	    case '.': fWorkPrn();
		      break;

	    case ',': SmeterPrn();
		      SmeterPrn();
		      SmeterPrn();
		      SmeterPrn();
		      SmeterPrn();
		      break;

	    case 'D': if(i)
		      {
			LcdWork = 1;
		      }
		      else
		      {
			/*
			LcdWork = 0;
			cli();
			LcdOut2(0);
			LcdOut2(0);
			sei();
			*/
		      }
		      break;

	    case 'H': wBufEEPROM (ADR_EE_Hi, (uint8_t *)&f, 4);
		      break;

	    case 'B': wBufEEPROM (ADR_EE_Lo, (uint8_t *)&f, 4);
		      break;


	    case 'F': SetFwork(f);
		      break;

	    case 'P': KeyScanF();
		      break;

	    case 'N': FlagBegin++;
		      KeyScanF();
		      break;

	    case 'V': wBufEEPROM (ADR_EE_vMax, (uint8_t *)&i, 1);
		      wBufEEPROM (ADR_EE_vWrk, (uint8_t *)&i, 1); break;
	    case 'v': wBufEEPROM (ADR_EE_vMin, (uint8_t *)&i, 1); break;

	   }
	   RxInd = 0;
	   TxInd = 0;
	   uart_putstr(PROMPT);
	}
     }
  }
  return;
}

///////////////////////////////
void UapyControl(uint8_t level)
{
   if(UapyFlg == 1)
   {
     SmeterUapy += level;

     if(++UapyCnt >= UapyNN)
     {
	SmeterUapy = SmeterUapy / UapyNN;

	if(SmeterUapy > UapyHi)
	{
	  if(Uapy > 1)
	  {
	     Uapy--; UapyOut();
	  }
	}
	if(SmeterUapy < UapyLo)
	{
	  if(Uapy < 255)
	  {
	     Uapy++; UapyOut();
	  }

	}
	SmeterUapy = 0;
	UapyCnt    = 0;
     }
   }
   return;
}

///////////////////
void VolumeUp(void)
{
   if(Pwm1 > Vol_Min)
   {
     OCR1A = Pwm1--;
   }
   Int2Lcd(Vol_Max-Pwm1,Lcd4i);  TimeClr = AUTO_CLR;
   return;
}

/////////////////////
void VolumeDown(void)
{
   if(Pwm1 < Vol_Max)
   {
     OCR1A = Pwm1++;
   }
   Int2Lcd(Vol_Max-Pwm1,Lcd4i);  TimeClr = AUTO_CLR;
   return;
}

///////////////////
void KeyVOL(void)
{
     if(Regim == RADIO)
     {
	Regim = SET_VOL;

	Str2Lcd(sVol);
	Int2Lcd(Vol_Max-Pwm1,Lcd4i);  TimeClr = AUTO_CLR;
     }
     return;
}

/////////////////////////////////////////////////////////////////////
void PDUvolUp  (void) { PduRADIO(); KeyVOL(); VolumeUp();   return; }
void PDUvolDown(void) { PduRADIO(); KeyVOL(); VolumeDown(); return; }

//////////////////////////////////////
//////////////////////////////////////

	  uint8_t l2aBuff[8];

//////////////////////////////////////
uint8_t * ul2a(uint32_t c,uint8_t len)
{
  uint8_t  i;
  uint8_t *p;

   p = &l2aBuff[len];
  *p = 0;

  for(i = len; i > 0; i--)
  {
      p--;
     *p  = c%10 +'0';
      c /= 10;
  }
  return p;
}

////////////////////////
uint32_t a2l(uint8_t *p)
{
  uint32_t l = 0;

  while(*p)
  {
     l *= 10;
     l += *p-'0';
     p++;
  }
  return l;
}

